# MyDBF2MySQL - Software for transferring DBF into MySQL.
# Copyright (C) 2008-2010 Polyakov Andrey
#
# http://sourceforge.net/projects/mydbf2mysql/
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, version 2
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

try:
    from threading import Thread
except ImportError:
    from dummy_threading import Thread
from MySQLdb import connect, Error, OperationalError, Warning
from counter import progressCounterST
from log import createLogger
from logging import INFO
from ydbf import YDbfReader
from ydbf.lib import DbfError


class workerThread(Thread):
    def __init__ (self, fileName, basename, dbpasswd, query, dbfIndex, startFromRecord=0,
                            stopAtRecord=0, dbhost="", dbuser="root", port=3306, connect_timeout=0,
                            compress=0, socket="", pipe="", connection=None, verboseLevel=INFO,
                            reconnect= -1, show_progress=False, pooling=64, strict=True,
                            encoding='ascii' , overwrite=False, unicode=False):
        self.connect_timeout = connect_timeout
        self.startFromRecord = startFromRecord
        self.stopAtRecord = stopAtRecord
        self.basename = basename
        self.dbpasswd = dbpasswd
        self.fileName = fileName
        self.dbfIndex = dbfIndex
        self.compress = compress
        self.reconnect = reconnect
        self.show_progress = show_progress
        self.connection = connection
        self.verboseLevel = verboseLevel
        self.pooling = pooling
        self.overwrite = overwrite
        self.encoding = encoding
        self.unicode = unicode
        self.socket = socket
        self.dbhost = dbhost
        self.dbuser = dbuser
        self.query = query
        self.strict = strict
        self.port = port
        self.pipe = pipe
        Thread.__init__ (self)

    def __connect(self):
        if self.socket:
            connection = connect(host=self.dbhost, user=self.dbuser, passwd=self.dbpasswd,
                                    db=self.basename, port=self.port, compress=self.compress,
                                    connect_timeout=self.connect_timeout, unix_socket=self.socket)
        else:
            connection = connect(host=self.dbhost, user=self.dbuser, passwd=self.dbpasswd,
                                    db=self.basename, port=self.port, compress=self.compress,
                                    connect_timeout=self.connect_timeout, named_pipe=self.pipe)
        return connection

    def run(self):
        treadLogger = createLogger(
                                   "thread",
                                   "%(levelname)s: %(message)s",
                                   self.verboseLevel)
        insertedByMe = failedToInsertByMe = 0
        globalCounter = progressCounterST()
        toProcess = self.stopAtRecord - self.startFromRecord
        query = self.query
        reconnectAttempts = self.reconnect
        try:
            dbfreader = YDbfReader(fh=file(self.fileName, 'rb'),
                                   strict=self.strict,
                                   use_unicode=self.unicode,
                                   overwrite_encoding=self.overwrite,
                                   default_encoding=self.encoding)
        except DbfError, (errno, strerror):
            treadLogger.error("Ydbf error occurred in %s: %s, %s." % \
                            (self.getName(), errno, strerror))
            if self.show_progress:
                globalCounter.increaseValues(toProcess, toProcess)
            return
        except IOError, (errno, strerror):
            treadLogger.error("Failed to open file in %s: %s, %s." % \
                            (self.getName(), errno, strerror))
            if self.show_progress:
                globalCounter.increaseValues(toProcess, toProcess)
            return
        except:
            treadLogger.error("%s: DBF file is invalid." % self.getName())
            if self.show_progress:
                globalCounter.increaseValues(toProcess, toProcess)
            return
        dbi = self.connection
        if dbi is None:
            try:
                dbi = self.__connect()
            except Error, (errno, strerror):
                treadLogger.error("MySQL error in %s: %s (%s)." % \
                                (self.getName(), strerror, errno))
                if self.show_progress:
                    globalCounter.increaseValues(toProcess, toProcess)
                return
        cursor = dbi.cursor()
        dbfreader.start_from = self.startFromRecord
        dbfreader.stop_at = self.stopAtRecord
        stop = self.pooling
        dataPool = []
        data = ()
        for record in dbfreader():
            if self.stopAtRecord - dbfreader.i < self.pooling:
                stop = 1
            stop = stop - 1
            for currentIndex, currentType in self.dbfIndex:
                value = record[currentIndex]
                if currentType is not None:
                    try:
                        value = currentType(value)
                    except:
                        value = 0
                data += (value,)
            dataPool.append(data)
            data = ()
            if stop < 1:
                stop = self.pooling
                while True:
                     try:
                         cursor.executemany(query, dataPool)
                     except Warning, strwarn:
                         treadLogger.warn(strwarn)
                         #filterwarnings('ignore', category=Warning)
                         # TO DO: Make it less noisy, but optionally.
                     except OperationalError, (errno, strerror):
                         treadLogger.error("%s disconnected from the server." % self.getName())
                         if OPT_RECNT < 0 or reconnectAttempts > 0:
                             while reconnectAttempts != 0:
                                 sleep(1)
                                 treadLogger.debug("%s  reconnection attempt: %s." % (self.getName(), reconnectAttempts))
                                 reconnectAttempts = reconnectAttempts - 1
                                 try:
                                     dbi = self.__connect()
                                 except OperationalError, (errno, strerror):
                                     continue
                                 cursor = dbi.cursor()
                                 treadLogger.info("%s got connected." % self.getName())
                                 break
                         else:
                             if self.show_progress:
                                globalCounter.increaseFailed(toProcess - failedToInsertByMe - insertedByMe)
                             return
                     except Error, (errno, strerror):
                         treadLogger.error("Unexpected MySQL error in %s: %s (%s)." % \
                         (self.getName(), strerror, errno))
                         if self.show_progress:
                            globalCounter.increaseFailed(toProcess - failedToInsertByMe - insertedByMe)
                         return
                     dataPool = []
                     if self.show_progress:
                        insertedByMe += cursor.rowcount
                        globalCounter.increaseProcessed(cursor.rowcount)
                     break
        if insertedByMe + failedToInsertByMe < toProcess:
            globalCounter.increaseFailed(toProcess - insertedByMe - failedToInsertByMe)
        cursor.close()
        dbi.close()
        return

